package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.dao.OrderItemDao;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.dto.OrderItemView;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.service.OrderItemService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class OrderItemServiceGenImpl implements OrderItemService {
    protected final Log logger = LogFactory.getLog(getClass());
    @Autowired(required=false)
    protected Mapper mapper;
    protected OrderItemDao orderItemDao;
    protected OrderDao orderDao;
    protected ProductDao productDao;
    
    @Autowired
    public void setOrderItemDao(OrderItemDao orderItemDao) {
        this.orderItemDao = orderItemDao;
    }

    @Autowired
    public void setOrderDao(OrderDao orderDao) {
        this.orderDao = orderDao;
    }

    @Autowired
    public void setProductDao(ProductDao productDao) {
        this.productDao = productDao;
    }
    
    @Transactional
	public OrderItemView save(OrderItemView orderItemView) {
        Assert.notNull(orderItemView, "parameter 'orderItemView' must not be null");
        OrderItem orderItem = this.mapper.createAndMapOne(orderItemView, OrderItem.class,"saveOrderItem");
        Order order = this.mapper.createAndMapOne(orderItemView, Order.class,"order");
        order = orderDao.load(order.getOid());
        orderItem.setOrder(order);
        Product product = this.mapper.createAndMapOne(orderItemView, Product.class,"product");
        product = productDao.load(product.getOid());
        orderItem.setProduct(product);
        this.orderItemDao.saveOrUpdate(orderItem);
        return this.mapper.mapOne(orderItem,new OrderItemView());
    }
    
    @Transactional(readOnly = true)
	public OrderItemView loadOrderItemView(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.orderItemDao.load(oid),OrderItemView.class);
    }
    
    @Transactional
	public void update(OrderItemView orderItemView) {
        Assert.notNull(orderItemView, "parameter 'orderItemView' must not be null");
        OrderItem mappedOrderItem = this.mapper.createAndMapOne(orderItemView, OrderItem.class);
        OrderItem orderItem = this.orderItemDao.load(mappedOrderItem.getOid());
        if (!orderItem.getVersion().equals(mappedOrderItem.getVersion())) {
            throw new OptimisticLockingFailureException("Entity 'OrderItem' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(orderItemView, orderItem,"updateOrderItem");
    }
    
    @Transactional
	public void deleteOrderItem(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        this.orderItemDao.delete(oid);
    }
        
    @Transactional
    public OrderItemView saveOrUpdate(OrderItemView orderItemView) {
        Assert.notNull(orderItemView, "parameter 'orderItemView' must not be null");
        if ( orderItemView.getVersion() == null ) {
            return this.save(orderItemView);
        }
        else {
            this.update(orderItemView);
            return orderItemView;
        }
    }

    
}
