package org.openxma.dsl.reference.xma.customeredit.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;
import at.spardat.xma.mdl.table.*;
import at.spardat.enterprise.fmt.*;

/**
 * This class implements the generic client part of Page2.
 * It implements the widgets and models of the page.
 *
 */
public abstract class Page2Gen extends org.openxma.dsl.platform.xma.client.DslNotebookPage {
    private Locale genPageLocale = null;
    private ResourceBundle genPageMessages = null;
    Composite orderTableHeaderW;
    Composite orderTableHeader_set0W;
    Label orderTableHeader_set0_centered;
    Label headerText;
    Table orderTableW;
    at.spardat.xma.mdl.table.TableLayoutManager orderTableWLM;
    ITableWMClient orderTable;
    static final int ORDERTABLE_PLACEMENTDATE = 0;
    static final int ORDERTABLE_ORDERSTATE = 1;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the Page2Gen.
     * @param notebook the Notebook this page belongs to.
     */
    public Page2Gen(Notebook notebook) {
        super(notebook,false);
        createModels();
    }

    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        if (genPageLocale == null) {
            genPageLocale = getComponent().getContext().getLocale();
        }
        return genPageLocale;
    }

    /*
     * returns the message resource bundle used for this page
    */
    final protected ResourceBundle getGenPageMessages() {
        if (genPageMessages == null) {
            genPageMessages = ResourceBundle.getBundle("org.openxma.dsl.reference.xma.customeredit.client.CustomerEdit",getGenPageLocale());
        }
        return genPageMessages;
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        IFmt formatter;
        orderTable = new TableWMClient((short) 0,this,2,TableWMClient.S_NULL|TableWM.S_ONE_WAY);
        formatter = ADateFmt.getInstance(ADateFmt.MEDIUM, getGenPageLocale());
        ((TableWMClient)orderTable).getColumn(ORDERTABLE_PLACEMENTDATE).setFormatter(formatter);
        formatter = AStringFmt.getInstance(33);
        ((TableWMClient)orderTable).getColumn(ORDERTABLE_ORDERSTATE).setFormatter(formatter);

        widgetModels = new WModel[] { (WModel) orderTable };
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        orderTable = null;
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return CustomerEdit.Page2Class;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public CustomerEdit getTypedComponent() {
        return (CustomerEdit) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        EventAdapter adapter = new EventAdapter(this);
        final Scaler scaler = Scaler.getInstance(getComposite());

        Composite pageCompositePage2W = getComposite();
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        pageCompositePage2W.setLayout(layout);

        orderTableHeaderW = new Composite(pageCompositePage2W,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        orderTableHeaderW.setLayout(layout);
        orderTableHeaderW.setData("WIDGET_ID","orderTableHeaderW");

        orderTableHeader_set0W = new Composite(orderTableHeaderW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(0);
        layout.marginWidth = scaler.convertXToCurrent(0);
        orderTableHeader_set0W.setLayout(layout);
        orderTableHeader_set0W.setData("WIDGET_ID","orderTableHeader_set0W");

        orderTableHeader_set0_centered = new Label(orderTableHeader_set0W,SWT.LEFT);
        orderTableHeader_set0_centered.setVisible(false);
        orderTableHeader_set0_centered.setData("WIDGET_ID","orderTableHeader_set0_centered");

        headerText = new Label(orderTableHeader_set0W,SWT.LEFT);
        headerText.setText(getGenPageMessages().getString("Page2.headerText"));
        headerText.setData("WIDGET_ID","headerText");

        orderTableW = new Table(pageCompositePage2W,SWT.SINGLE|SWT.BORDER|SWT.FULL_SELECTION
);
        orderTableW.setHeaderVisible(true);
        orderTableW.setLinesVisible(true);
        orderTableW.setData("WIDGET_ID","orderTableW");
        orderTableW.addSelectionListener(adapter);
        TableColumn orderTable_placementDate = new TableColumn(orderTableW,SWT.LEFT);
        orderTable_placementDate.setText(getGenPageMessages().getString("Page2.orderTable_placementDate"));
        orderTable_placementDate.addSelectionListener(adapter);
        TableColumn orderTable_orderState = new TableColumn(orderTableW,SWT.LEFT);
        orderTable_orderState.setText(getGenPageMessages().getString("Page2.orderTable_orderState"));
        orderTable_orderState.addSelectionListener(adapter);
        orderTableW.addMouseListener(adapter);
        Menu orderTableWCM = new Menu(orderTableW.getShell());
        MenuItem orderTableWCMI = new MenuItem(orderTableWCM, SWT.PUSH);
        orderTableWCMI.setText("Copy text");
        orderTableWCMI.addSelectionListener(adapter);
        orderTableW.setMenu(orderTableWCM);
        orderTableWLM = new at.spardat.xma.mdl.table.TableLayoutManager();
        orderTableWLM.manageTable(orderTableW,at.spardat.xma.mdl.table.TableLayoutManager.NONE);
        orderTableWLM.setMinLimit(scaler.convertXToCurrent(10));
        orderTableWLM.setAbsolutWidth(0,scaler.convertXToCurrent(80));
        orderTableWLM.setAbsolutWidth(1,scaler.convertXToCurrent(140));

        widgets = new Control[] { orderTableHeaderW, orderTableHeader_set0W, orderTableHeader_set0_centered, headerText, orderTableW };

        createWidgetsLayout();
        createWidgetsTabOrder();

    }

    /**
     * Create the layout of all SWT-Widgets of the page.
     * (Attachments, Distances, Sizes)
     */
    public void createWidgetsLayout() {
        FormData data;
        final Scaler scaler = Scaler.getInstance(getComposite());
        Control lastControl;
        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.bottom = new FormAttachment(0,100,scaler.convertYToCurrent(20));
        orderTableHeaderW.setLayoutData(data);

        lastControl=null;
        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
            data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        orderTableHeader_set0W.setLayoutData(data);
        lastControl=orderTableHeader_set0W;

        data = new FormData();
        data.width = scaler.convertXToCurrent(1);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        orderTableHeader_set0_centered.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(orderTableHeader_set0_centered,scaler.convertYToCurrent(0),SWT.CENTER);
        headerText.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(orderTableHeaderW,scaler.convertYToCurrent(0),SWT.BOTTOM);
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        orderTableW.setLayoutData(data);

    }

    /**
     * Create the tab order of all SWT-Widgets of the page.
     * (sets the focus and defines the tab order)
     */
    protected void createWidgetsTabOrder() {
        Control[] tabOrder;
        final Composite pageCompositePage2W = getComposite();
        tabOrder = new Control[2];
        tabOrder[0] = orderTableHeaderW;
        tabOrder[1] = orderTableW;
        pageCompositePage2W.setTabList(tabOrder);
        tabOrder = new Control[1];
        tabOrder[0] = orderTableHeader_set0W;
        orderTableHeaderW.setTabList(tabOrder);
        tabOrder = new Control[0];
        orderTableHeader_set0W.setTabList(tabOrder);
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        orderTableHeaderW = null;
        orderTableHeader_set0W = null;
        orderTableHeader_set0_centered = null;
        headerText = null;
        orderTableW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
        TableColumn[] cols;
        orderTableW.setData(orderTable.getUIDelegate());
        orderTable.getUIDelegate().attachUI(orderTableW,null);
        cols = orderTableW.getColumns();
        for(int i = 0; i<cols.length; i++) {
            cols[i].setData(orderTable.getUIDelegate());
        }
        if (orderTableW.getMenu() != null) {
           for(MenuItem item : orderTableW.getMenu().getItems()) {
               item.setData(orderTable.getUIDelegate());
           }
        }

    }

    /**
     * This method will be called every time the Page2 becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == orderTableW  && type == SWT.Selection) {
            ((CustomerEditForm)getParent()).selectOrder(event);
        }
    }


    /**
     * Creates the coresponding TabItem
     */
    protected TabItem createTabItem(TabFolder tabfolder1W,int index) {
        TabItem page2W = new TabItem (tabfolder1W, SWT.NULL, index);
        page2W.setText(getGenPageMessages().getString("Page2"));
        page2W.setControl(getComposite());
        page2W.setData(this);

        return page2W;
    }
}
